;**************************************************************************************************************************************************
;   LIBRARY:      CICLI DI RITARDO PER CLOCK A 32MHz
;   FILE:         Delay_32MHz.asm
;   LAST REV:     1.1
;	AUTHOR:       Ivo Colleoni
;  	DEVICE:       PIC18F2550
;	CREATED:	  09/02/2013
;	UPDATED:	  04/04/2013
;
;	DESCRIP:      Crea cicli di ritardo, usare con clock a 32Mhz
;
;   FUNCTION:     DELAY_180us
;                 DELAY_2ms
;                 DELAY_25ms
;                 DELAY_100ms
;                 DELAY_250ms
;                 DELAY_Xms
;                 RESET_VAR_DELAY
;
;--------------------------------------------------------------------------------------------------------------------------------------------------
;   Change History:
;  	Rev.    Date                Changes
;   1.1     04/04/2013          - aggiunto DELAY_100ms
;   1.0     09/02/2013          - prima build
;
;**************************************************************************************************************************************************
;dichiarazione variabili, memoria allocata da 0x04 a 0x07
                    CBLOCK              0x04
                    delay_1
                    delay_2
					delay_3
                    delay_x
                    ENDC
;
;**************************************************************************************************************************************************
;   FUNCTION:       DELAY_180us
;   Overview:       ritardo di 180uS @32MHz (utilizzato per la scrittura su display)
;**************************************************************************************************************************************************
;
DELAY_180us        	movlw               h'03'
                    movwf               delay_2

DELAY_180us_1       movlw               h'9E'
                    movwf               delay_1

DELAY_180us_2       decfsz              delay_1,f
                    goto                DELAY_180us_2

                    decfsz              delay_2,f
                    goto                DELAY_180us_1

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       DELAY_2ms
;   Overview:       ritardo di 2mS @32MHz (lunog, utilizzato per la scrittura su display)
;**************************************************************************************************************************************************
;
DELAY_2ms           movlw               h'17'
                    movwf               delay_2

DELAY_2ms1          movlw               h'E7'
                    movwf               delay_1

DELAY_2ms2          decfsz              delay_1,f
                    goto                DELAY_2ms2

                    decfsz              delay_2,f
                    goto                DELAY_2ms1

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       DELAY_25ms
;   Overview:       ritardo di 25mS @32MHz
;**************************************************************************************************************************************************
;
DELAY_25ms			movlw		h'4'
					movwf		delay_3
DELAY_25ms_1		movlw		h'82'
					movwf		delay_2
DELAY_25ms_2		movlw		h'7F'
					movwf		delay_1

DELAY_25ms_3		decfsz		delay_1,F
					goto		DELAY_25ms_3
						
					decfsz		delay_2,F
					goto		DELAY_25ms_2

					decfsz		delay_3,F
					goto		DELAY_25ms_1

					return
;
;**************************************************************************************************************************************************
;   FUNCTION:       DELAY_100ms
;   Overview:       ritardo di 100mS @32MHz (DELAY_25ms x4)
;**************************************************************************************************************************************************
;
DELAY_100ms	  		movlw				d'4'
					movwf               delay_x
DELAY_100ms_1      	call                DELAY_25ms
                    decfsz              delay_x,F
                    goto                DELAY_100ms_1

                   	return
;
;**************************************************************************************************************************************************
;   FUNCTION:       DELAY_250ms
;   Overview:       ritardo di 250mS @32MHz (DELAY_25ms x10)
;**************************************************************************************************************************************************
;
DELAY_250ms	  		movlw				d'10'
					movwf               delay_x
DELAY_250ms_1      	call                DELAY_25ms
                    decfsz              delay_x,F
                    goto                DELAY_250ms_1

                   	return
;
;**************************************************************************************************************************************************
;   FUNCTION:       DELAY_Xms
;   Input:          WREG
;   Overview:       ripete n volte DELAY_25ms (n=W) es W=d'40' ritardo di circa 1 secondo  W=d'120' ritardo di circa 3 secondi
;**************************************************************************************************************************************************
;
DELAY_Xms	  		movwf               delay_x
DELAY_Xms_1        	call                DELAY_25ms
                    decfsz              delay_x,F
                    goto                DELAY_Xms_1

                   	return
;
;**************************************************************************************************************************************************
;   FUNCTION:       RESET_VAR_DELAY
;   Overview:       Resetta tutte le variabili utilizzate nei ritardi
;**************************************************************************************************************************************************
;
RESET_VAR_DELAY     clrf                delay_1
                    clrf                delay_2
					clrf                delay_3
                    clrf                delay_x
                    return