;**************************************************************************************************************************************************
;   PROJECT:      GENERATORE DI FUNZIONI DSS
;   FILE:         Main.asm
;   LAST REV:     2.5
;	AUTHOR:       Ivo Colleoni
;  	DEVICE:       PIC18F2550
;	CREATED:	  15/01/2013
;	UPDATED:	  07/06/2013
;
;	DESCRIP:      Programma principale.
;--------------------------------------------------------------------------------------------------------------------------------------------------
;   Change History:
;  	Rev.    Date                Changes
;   2.5     07/06/2013          - inserito controllo causa reset con esposizione su display e salvataggio in EEPROM
;                               - bug fixes
;   2.4     30/04/2013          - gestione vpp con multiplexer (cambiati tutti i cicli VPP)
;                               - vpp vale da 1 a 8 per i valori sotto 1V mentre vale da 11 a 18 per i valori sopra l'1V. il bit 4 indica se il
;                                 rel va eccitato (bit4=0) o no (bit4=1)
;                               - grestione vpp separata per sinusoidi e onde quadre
;                               - inserita possibilit di scelta tra 2 swap: 1hz->1Mhz e 20Hz->48Khz
;   2.3     22/04/2013          - gestione separata offset per onde sinusoidali/triangolari e onde quadre, pot_offset  diventata variabile globale per
;                                 la programmazione SPI del potenziometro, mentre per i calcoli e le esposizioni sono state inserite 2 varabili:
;                                 pot_offset_sin e pot_offset_squ
;                               - inserito reset del DSS all'avvio
;   2.2     06/04/2013          - Bug Fixes
;                               - Potenziometro digitale per il guadagno cabiato con MCP41010, impostato VPP_MIN a 03 e partitore attivo fino a 1V
;                                 per ripristinare MCP41010 impostare VPP_MIN a 02 e seguire commenti in INC_VPP e VPP_TO_POT
;   2.1     05/04/2013          - Implementata gestione offset
;                               - Modificate posizioni scritte su LCD
;   2.0     30/03/2013          - Implementata gestione Vpp da 1 a 4vpp in step da 0.2 e da 4 a 5vpp in 4 step fissi con gestione uscita rele per
;                                 partitore finale (valori minori a 1V).
;                               - Implementato swap da 0 a 1 Mhz
;                               - Implementata impostazioni valori standard al primo avvio dopo la programmazione
;   1.2     03/03/2013          - migliorata gestione pulsanti
;   1.1     26/02/2013          - menu ridisegnato
;                               - implementata gestione della fase
;   1.0     20/02/2013:         - prima build
;                               - Selezione forma d'onda (SIN-TRI-SQU)
;                               - Impostazione frequenza 0-8.000.000 Hz (+-0.03hz)
;                               - gestione contrasto e retroilluminazione in PWM
;                               - caricamento dati salvati all'avvio (frequenza, forma, retroilluminazione e contrasto)
;                               - Menu su 3 pagine: pagina 1 impostazioni paramenti onda attuale, pagina 2: retroilluminazione e contrasto,
;                               - pagina 3 modifica valori all'avvio
;                               - simulazione protocollo SPI
;
;**************************************************************************************************************************************************
;CONNESSIONI DEI PIN
;**************************************************************************************************************************************************
;
;RA0: Up
;RA1: Down
;RA2: Left
;RA3: Right
;RA4: Enter
;RA5: 
;RA6: OSC
;
;RB0: DB4
;RB1: DB5
;RB2: DB6
;RB3: DB7
;RB4: RS
;RB5: E
;RB6: FSYNC_POT_OFF
;RB7: FSYNC_MUX_VPP
;
;RC0: FSYNC AD9833
;RC1: PWM2 (retroilluminazione)
;RC2: PWM1 (contrasto)
;RC4: x
;RC5: x
;RC6: SPI DATA
;RC7: SPI CLOCK
;
;**************************************************************************************************************************************************
;MAPPATURA EEPROM
;**************************************************************************************************************************************************
;
;0x00   duty cicle PWM1 (contrasto)
;0x01   duty cicle PWM2 (retroilluminazione)
;0x02   shape
;0x03   freq_ins_0
;0x04   freq_ins_1
;0x05   freq_ins_2
;0x06   freq_ins_3
;0x07   freq_ins_4
;0x08   freq_ins_5
;0x09   freq_ins_6
;0x0A   phase_h
;0x0B   vpp
;0x0C   pot_offset_sin
;0X0D   pot_offset_squ
;0x0E   first_start
;0x10   reset_status
;
;**************************************************************************************************************************************************
;DIRETTIVE AL COMPILATORE
;**************************************************************************************************************************************************
;
            		processor	18F2550
            		radix		DEC
            		include		"P18F2550.inc"

                    CONFIG PLLDIV = 1                                                   ;No prescaler (4 MHz oscillator input comanda il PLL direttamente)
                    CONFIG CPUDIV = OSC2_PLL3                                           ;96 MHz PLL/3 = 32MHz
                    CONFIG FOSC = HSPLL_HS    	                                        ;Clock esterno con PLL
                   	CONFIG PBADEN = OFF    		                                        ;PORTB <4:0> come digital I/O
            		CONFIG DEBUG = OFF     		                                        ;Debug/prog su RB6 e RB7
                    CONFIG LVP = OFF
            		CONFIG PWRT = OFF
            		CONFIG MCLRE = ON
                    CONFIG WDT = OFF
            		CONFIG CP0 = OFF
            		CONFIG CP1 = OFF
            		CONFIG CPD = OFF
                    CONFIG BOR = OFF                                                    ;Bit di abilitazione del Brown-out Reset
                    CONFIG STVREN = ON                                                  ;Abilitazione del reset da Stack Full/Underflow
;
;**************************************************************************************************************************************************
;DEFINIZIONE NOMI IN/OUT
;**************************************************************************************************************************************************
;
            		#DEFINE     PORT_LCD        LATB
                    #DEFINE     LCD_E           LATB,5
                    #DEFINE     LCD_RS          LATB,4
            		#DEFINE		UP              PORTA,0
            		#DEFINE		DOWN            PORTA,1
                    #DEFINE     LEFT            PORTA,2
                    #DEFINE     RIGHT           PORTA,3
            		#DEFINE		ENTER           PORTA,4
                    #DEFINE     SPI_D           LATC,6
                    #DEFINE     SPI_CK          LATC,7
                    #DEFINE     FSYNC_DSS       LATC,0
                    #DEFINE     FSYNC_POT_OFFS  LATB,6
                    #DEFINE     FSYNC_MUX_VPP   LATB,7
;
;**************************************************************************************************************************************************
;COSTANTI
;**************************************************************************************************************************************************
;
                    CONSTANT    LCD_UNUSED  = b'11000000'                               ;RB6, RB7 non usati per la gestione dell'LCD
            		CONSTANT	TIME_INFO   = d'120'                                    ;usato per l'esposizione temporizzata delle informazioni iniziali
                    CONSTANT    FREQ_MAX    = h'08'

                    CONSTANT    HW_REV_0    = d'3'                                      ;rev. hardware
                    CONSTANT    HW_REV_1    = d'0'
                    CONSTANT    HW_REV_2    = d'0'
                    CONSTANT    FW_REV_0    = d'2'                                      ;rev. firmware
                    CONSTANT    FW_REV_1    = d'5'
                    CONSTANT    FW_REV_2    = d'0'

                    CONSTANT    EE_first_start      = h'0E'                             ;indirizzi in memoria EEPROM
                    CONSTANT    EE_duty_contrast    = h'00'
                    CONSTANT    EE_duty_backill     = h'01'
                    CONSTANT    EE_pot_offset_sin   = h'0C'
                    CONSTANT    EE_pot_offset_squ   = h'0D'
                    CONSTANT    EE_reset_status     = h'10'
;
;**************************************************************************************************************************************************
;VARIABILI
;**************************************************************************************************************************************************
;
					udata
freq_ins_0			res			1			;LSB           								;Frequenza isnerita
freq_ins_1			res			1
freq_ins_2			res			1
freq_ins_3			res			1
freq_ins_4			res			1
freq_ins_5			res			1
freq_ins_6			res			1

freq_hex_s_0        res         1			;LSB										;valore frequenza esadecimale
freq_hex_s_1        res         1
freq_hex_s_2        res         1
freq_hex_s_3        res         1

count               res         1
temp				res			1
shape               res         1                                                       ;0=quadra, 1=trian 2=sin

dss_ctrl_l          res         1
dss_ctrl_h          res         1

phase_h             res         1
vpp                 res         1                                                       ;valori da 1 a 8 per vpp da 0.2 a 1 e da 11 a 18 per vpp da 1.5 a 10
mux_vpp             res         1

pot_offset_sin      res         1
pot_offset_squ      res         1
pot_offset          res         1

reset_status        res         1
;
;**************************************************************************************************************************************************
;   INIZIO PROGRAMMA PRINCIPALE
;**************************************************************************************************************************************************
;
                    org                 0x00
                    goto                MAIN                                            ;salto il blocco di cestine interupt

                    org					0x18
MAIN
;
;**************************************************************************************************************************************************
;   DEFINIZIONI I/O
;**************************************************************************************************************************************************
;
    				clrf			    PORTA									        ;
					movlw				0Fh 											;Configuro A/D
					movwf 				ADCON1 											;per input digitali
					movlw				07h 											;Configuro comparatori
					movwf				CMCON 											;per input digitali
					movlw 				b'00011111'                                     ;RA0-RA4 input, RA5 output
					movwf 				TRISA

                    clrf			    PORTB
					movlw 				b'00000000'                                     ;RB0-RB7 output
					movwf 				TRISB

                    clrf			    PORTC
                    movlw 				b'00000000'                                     ;RC0-RC7 output
					movwf 				TRISC

					bcf					UCON,3                                          ;disabilito USB
					bsf					UCFG,3
;
;**************************************************************************************************************************************************
;   FUNCTION:       INIZIALIZZAZIONE
;   Overview:       Resetta il dss disabilitando l'uscita, resetta tutte le variabili, avvia l'lcd, i moduli PWM e SPI
;                   controlla se il programma non  mai stato eseguito e richiama la funzione FIRST_START.
;**************************************************************************************************************************************************
;
                    call				RESET_VAR_MAIN                                  ;reset di tutte le variabili
                    call                RESET_VAR_LCD
                    call                RESET_VAR_SPI
                    call                RESET_VAR_DELAY
                    call                RESET_VAR_MEMORY
                    call                RESET_VAR_CALC

                    call                SPI_START                                       ;imposto l'SPI ai valori iniziali (SYNC=1 CK=1 D=0)

                    movlw               b'00000001'                                     ;abilito il reset (D9:1) del DSS
                    movwf               dss_ctrl_h                                      ;
                    call                SPI_SEND_CTRL                                   ;
                	
                    call                LOAD_START_VALUE                                ;Carico i valori salvati in EEPROM nelle rispettive variabili
                    call                PWM_START                                       ;Avvio i moduli PWM
                    call				DELAY_25ms                                      ;Delay 25ms per stabilizzare il Display
					call               	LCD_INIT 		                                ;Inizializzo il Display

                    movlw               EE_first_start                                  ;leggo il flag di primo caricamento dalla EEPROM
                    movwf               ee_adr
                    call                READ_EE

                    movf                ee_data,W                                       ;se dato letto da EEPROM = 01 il programma  gia stato avviato
                    xorlw               h'01'                                           ;una volta
                    btfss               STATUS,Z                                        ;quindi salto
                    call                FIRST_START                                     ;altrimenti vado a FIRST_START per impostare valori standard sia in locale che in EEPROM

;                   call                TEST_RESET                                      ;controllo causa reset, decommentare in caso di reset anomali
;
;**************************************************************************************************************************************************
;TEST
;**************************************************************************************************************************************************
;
;**************************************************************************************************************************************************
;   FUNCTION:       START
;   Overview:       espone i valori iniziali, calcola la prima frequenza e manda i dati iniziali DSS e al potenziometro e ai multiplezer mantenedo
;                   il DSS in reset fino a caricamento completato
;**************************************************************************************************************************************************
;
START               call                EXPO_NAME                                       ;Espongo il nome
					movlw				TIME_INFO                                       ;Ritardo esposizione
                    call                DELAY_Xms
                    call                EXPO_INFO                                       ;Espongo informazioni
					movlw				TIME_INFO
                    call                DELAY_Xms

                    call                EXPO_STATIC                                     ;Espongo le scritte statiche su display
                    call                EXPO_SHAPE                                      ;Espongo la forma d'onda
                    call                EXPO_FREQ                                       ;Espongo la frequenza
                    call                EXPO_PHASE                                      ;Espongo la fase
                    call                EXPO_VPP                                        ;Espongo vpp
                    call                EXPO_SWAP                                       ;Espongo la scritta SWAP

                    call				MOLTIP                                          ;moltiplico la frequenza caricata per M
 					call				BCD9_TO_HEX4                                    ;converto il risultato in 4 byte esadecimali
                    call                SHAPE_TO_CTRL                                   ;imposto dss_ctrl_l a seconda della forma d'onda selezionata
                    call                VPP_TO_MUX                                      ;imposto il valore da mandare al multiplrexer (salva il valore in mux_vpp)

                    movlw               b'00100001'                                     ;imposto dss_ctrl_h per mandare una frequenza completa (4byte) e abilito il reset
                    movwf               dss_ctrl_h                                      ;D15-D14:00 (control register) ;D13:1 (2 operazioni di scrittura) e D9:1 (reset)
                    call                SPI_SEND_CTRL                                   ;mando forma d'onda e avviso di programmazione frequenza  (dss_ctrl_h e dss_ctrl_l al DSS)

                    call                SPI_SEND_FREQ                                   ;mando i dati relativi alla frequenza al DSS
                    call                SPI_SEND_PHASE                                  ;mando i dati relativi alla fase al DSS
                    call                SPI_SEND_MUX_VPP                                ;programmo il multiplexer per abilitare lo switch relativo

                    movff               pot_offset_sin, pot_offset                      ;imposto l'offset per la sinusoide nell'offset da programmare

                    movf                shape,W                                         ;se shape = 0
                    btfsc               STATUS,Z
                    movff               pot_offset_squ, pot_offset                      ;imposto l'offset del'onda quandra nell'offset da programmare altrimenti resta quello della sinusoide

                    call                SPI_SEND_POT_OFFS                               ;programmp il potenziometro digitale per l'offset con il dato contenuto in pot_offset

                    movlw               b'00000000'                                     ;tolgo il reset
                    movwf               dss_ctrl_h

                    call                SPI_SEND_CTRL                                   ;azzero dss_ctrl_h mentre in dss_ctrl_l ho sempre il dato della forma d'onda
;
;**************************************************************************************************************************************************
;   FUNCTION:       HOME
;   Overview:       una volta mandato il primo dato al DSS controllo ciclicamente la pressione di ENTER e RIGHT per andare rispettivamente al menu
;                   o allo swap
;**************************************************************************************************************************************************
;
HOME                btfsc               ENTER                                           ;ENTER = 1
                    call                MENU_P1                                         ;vado al menu impostazioni

                    btfsc               RIGHT                                           ;RIGHT = 1
                    call                MENU_SWAP                                       ;vado a MENU_SWAP

                    goto                HOME                                            ;loop in HOME
;
;**************************************************************************************************************************************************
;   FUNCTION:       MENU
;   Overview:       Ggestisce lo spostamento del cursore nelle varie posizioni del menu e richiama i settaggi se richiesto
;**************************************************************************************************************************************************
;--------------------------------------------------------------------------------------------------------------------------------------------------
;Gestione spostamento menu PAGINA 1
;--------------------------------------------------------------------------------------------------------------------------------------------------
;esposizione valori iniziali e scritte
MENU_P1             call                RELEASE_T                                       ;Attendo che vengano rilasciati tutti i tasti (ritorno da pag 2)
                    call                EXPO_STATIC                                     ;espongo ancora la parte statica e i valori impostati
                    call                EXPO_SHAPE                                      ;serve se torno da menu successivo
                    call                EXPO_FREQ
                    call                EXPO_PHASE
                    call                EXPO_VPP
                    call                EXPO_OK_NEXT                                    ;espongo le scritte OK e MENU

                    movlw               d'0'                                            ;mi sposto in posizione 0 della prima riga
                    call                LCD_MOVE_LOC_R1

                    movlw               LCD_CURSOR_ON                                   ;e attivo la linea cursore
                    call                LCD_SEND_CODE
;--------------------------------------------------------------------------------------------------------------------------------------------------
;cursore su forma d'onda
SEL_P1_SHAPE        call                RELEASE_E                                       ;attendo il rilascio del tasto ENTER (ritorno da SETUP_SHAPE)
                    movlw               d'0'                                            ;sposto il cursore sulla forma d'onda
                    call                LCD_MOVE_LOC_R1
                    call                DELAY_250ms                                     ;ritardo
SEL_P1_SHAPE_T      btfsc               UP                                              ;UP=1
                    goto                SEL_P1_VPP                                      ;sposto il cursore su VPP
                    btfsc               DOWN                                            ;DOWN=1
                    goto                SEL_P1_FREQ                                     ;sposto il cursore sulla frequenza
                    btfsc               LEFT                                            ;LEFT=1
                    goto                MENU_EXIT                                       ;esco dal SETUP
                    btfsc               RIGHT                                           ;RIGHT=1
                    goto                MENU_P2                                         ;vado alla seconda pagina del menu
                    btfss               ENTER                                           ;ENTER=1 salta
                    goto                SEL_P1_SHAPE_T                                  ;altrimenti ricontrollo i tasti

                    call                DELETE_OK_NEXT                                  ;cancello le scritte ok e next
                    movlw               d'8'                                            ;sposto il cursore sul valore della forma d'onda
                    call                LCD_MOVE_LOC_R1
                    call                SETUP_SHAPE                                     ;imposto la forma d'onda
                    call                EXPO_VPP                                        ;espongo il vpp, potrebbe cambiare se cambia la forma d'onda da sinusoidale a quadra e vice versa

                    call                EXPO_OK_NEXT                                    ;al ritorno espongo nuovamente le scritte OK e NEXT
                    goto                SEL_P1_SHAPE                                    ;torno a SEL_P1_SHAPE (resto su questa selezione)
;--------------------------------------------------------------------------------------------------------------------------------------------------
;cursore su frequenza
SEL_P1_FREQ         call                RELEASE_E                                       ;attendo il rilascio del tasto ENTER (ritorno da SETUP_FREQ)
                    movlw               d'0'                                            ;sposto il cursore sulla frequenza
                    call                LCD_MOVE_LOC_R2
                    call                DELAY_250ms                                     ;ritardo

SEL_P1_FREQ_T       btfsc               UP                                              ;UP=1
                    goto                SEL_P1_SHAPE                                    ;Sposto il cursore sulla forma d'onda
                    btfsc               DOWN                                            ;DOWN=1
                    goto                SEL_P1_PHASE                                    ;sposto il cursore sulla fase
                    btfsc               LEFT                                            ;LEFT=1
                    goto                MENU_EXIT                                       ;esco dal SETUP
                    btfsc               RIGHT                                           ;RIGHT=1
                    goto                MENU_P2                                         ;vado alla seconda pagina del menu
                    btfss               ENTER                                           ;ENTER=1 salta
                    goto                SEL_P1_FREQ_T                                   ;altrimenti ricontrollo i tasti


                    call                EXPO_ARROW                                      ;espongo le frecce per la programmazione della frequenza
                    movlw               d'15'                                           ;sposto il cursore sul primo valore della frequenza
                    call                LCD_MOVE_LOC_R2
                    call                SETUP_FREQ                                      ;vado ad impostare la frequenza
                    call                EXPO_OK_NEXT                                    ;al ritorno espongo nuovamente le scritte OK e NEXT

                    goto                SEL_P1_FREQ                                     ;torno a SEL_P1_FREQ
;--------------------------------------------------------------------------------------------------------------------------------------------------
;cursore su fase
SEL_P1_PHASE        call                RELEASE_E                                       ;attendo il rilascio del tasto ENTER (ritorno da SETUP_PHASE)
                    movlw               d'0'                                            ;sposto il cursore sulla fase
                    call                LCD_MOVE_LOC_R3
                    call                DELAY_250ms                                     ;ritardo
SEL_P1_PHASE_T      btfsc               UP                                              ;UP=1
                    goto                SEL_P1_FREQ                                     ;sposto il cursore sulla frequenza
                    btfsc               DOWN                                            ;DOWN=1
                    goto                SEL_P1_VPP                                      ;sposto il cursore su VPP
                    btfsc               LEFT                                            ;LEFT=1
                    goto                MENU_EXIT                                       ;esco dal SETUP
                    btfsc               RIGHT                                           ;RIGHT=1
                    goto                MENU_P2                                         ;vado alla seconda pagina del menu
                    btfss               ENTER                                           ;ENTER=1 salta
                    goto                SEL_P1_PHASE_T                                  ;altrimenti ricontrollo i tasti

                    call                DELETE_OK_NEXT                                  ;elimino le scritte del menu
                    movlw               d'8'                                            ;sposto il cursore sul valore della fase
                    call                LCD_MOVE_LOC_R3
                    call                SETUP_PHASE                                     ;vado ad impostare la fase
                    call                EXPO_OK_NEXT                                    ;al ritorno espongo nuovamente le scritte OK e NEXT

                    goto                SEL_P1_PHASE                                    ;torno a SEL_P1_PHASE
;--------------------------------------------------------------------------------------------------------------------------------------------------
;cursore su vpp
SEL_P1_VPP          call                RELEASE_E                                       ;attendo il rilascio del tasto ENTER (ritorno da SETUP_VPP)
                    movlw               d'0'                                            ;sposto il cursore su Vpp
                    call                LCD_MOVE_LOC_R4
                    call                DELAY_250ms                                     ;ritardo

SEL_P1_VPP_T        btfsc               UP                                              ;UP=1
                    goto                SEL_P1_PHASE                                    ;sposto il cursore sulla fase
                    btfsc               DOWN                                            ;DOWN=1
                    goto                SEL_P1_SHAPE                                    ;sposto il cursore sulla forma d'onda
                    btfsc               LEFT                                            ;LEFT=1
                    goto                MENU_EXIT                                       ;esco dal SETUP
                    btfsc               RIGHT                                           ;RIGHT=1
                    goto                MENU_P2                                         ;vado alla seconda pagina del menu
                    btfss               ENTER                                           ;ENTER=1 salta
                    goto                SEL_P1_VPP_T                                    ;altrimenti ricontrollo i tasti

                    call                DELETE_OK_NEXT                                  ;elimino le scritte del menu
                    movlw               d'9'                                            ;sposto il cursore sul valore del Vpp
                    call                LCD_MOVE_LOC_R4
                    call                SETUP_VPP                                       ;vado ad impostare il Vpp
                    call                EXPO_OK_NEXT                                    ;al ritorno espongo nuovamente le scritte OK e NEXT
                    goto                SEL_P1_VPP                                      ;torno a SEL_P1_VPP
;
;--------------------------------------------------------------------------------------------------------------------------------------------------
;Gestione spostamento menu PAGINA 2
;--------------------------------------------------------------------------------------------------------------------------------------------------
;esposizione valori iniziali e scritte
MENU_P2             call                RELEASE_T                                       ;attendo che vengano rilasciati tutti i tasti
                    call                EXPO_PAGE_2                                     ;espongo i valori statici della seconda pagina
                    call                EXPO_BK_NEXT                                    ;espongo le scritte BK e NEXT

                    call                EXPO_BACKILL                                    ;espongo i valori di retroilluminazione e contrasto
                    call                EXPO_CONTR
                    call                EXPO_OFFSET_SIN                                 ;espongo l'offset
                    call                EXPO_OFFSET_SQU

;--------------------------------------------------------------------------------------------------------------------------------------------------
;cursore su retro illuminazione
SEL_P2_BACKILL      call                RELEASE_E
                    movlw               d'0'                                            ;sposto il cursore sulla retro illuminazione
                    call                LCD_MOVE_LOC_R1
                    call                DELAY_250ms
SEL_P2_BACKILL_T    btfsc               UP                                              ;UP=1
                    goto                SEL_P2_OFFSET_SQU                               ;sposto il cursore sull'offset per l'onda quadra
                    btfsc               DOWN                                            ;DOWN=1
                    goto                SEL_P2_CONTR                                    ;sposto il cursore sul contrasto
                    btfsc               LEFT                                            ;LEFT=1
                    goto                MENU_P1                                         ;torno alla prima pagina del menu
                    btfsc               RIGHT                                           ;RIGHT=1
                    goto                MENU_P3                                         ;vado alla terza pagina del menu
                    btfss               ENTER                                           ;ENTER=1 salto
                    goto                SEL_P2_BACKILL_T                                ;altrimenti ricontrollo i tasti

                    call                DELETE_OK_NEXT                                  ;cancello le scritte BK e NEXT
                    movlw               d'12'                                           ;sposto il cursore sul valore della retro illuminazione
                    call                LCD_MOVE_LOC_R1
                    call                SETUP_BACKILL                                   ;imposto la retro illuminazione
                    call                EXPO_BK_NEXT                                    ;al tirotno espongo di nuovo le scritte BK e NEXT
                    goto                SEL_P2_BACKILL                                  ;torno a SEL_P2_BACKILL
;--------------------------------------------------------------------------------------------------------------------------------------------------
;cursore su contrasto
SEL_P2_CONTR        call                RELEASE_E
                    movlw               d'0'                                            ;sposto il cursore sul contrasto
                    call                LCD_MOVE_LOC_R2
                    call                DELAY_250ms
SEL_P2_CONTR_T      btfsc               UP                                              ;UP=1
                    goto                SEL_P2_BACKILL                                  ;Sposto il cursore sulla retro illuminazione
                    btfsc               DOWN                                            ;DOWN=1
                    goto                SEL_P2_OFFSET_SIN                               ;sposto il cursore sull'offset per l'onda sinusoidale
                    btfsc               LEFT                                            ;LEFT=1
                    goto                MENU_P1                                         ;vado alla terza pagina del menu
                    btfsc               RIGHT                                           ;RIGHT=1
                    goto                MENU_P3                                         ;vado alla terza pagina del menu
                    btfss               ENTER                                           ;ENTER=1 salto
                    goto                SEL_P2_CONTR_T                                  ;altrimenti ricontrollo i tasti

                    call                DELETE_OK_NEXT                                  ;cancello le scritte BK e NEXT
                    movlw               d'12'                                           ;sposto il cursore sul valore del contrasto
                    call                LCD_MOVE_LOC_R2
                    call                SETUP_CONTR                                     ;imposto il contrasto
                    call                EXPO_BK_NEXT                                    ;al tirotno espongo di nuovo le scritte BK e NEXT
                    goto                SEL_P2_CONTR                                    ;torno a SEL_P2_CONTR
;--------------------------------------------------------------------------------------------------------------------------------------------------
;cursore su offset per onda sinusoidale e triangolare
SEL_P2_OFFSET_SIN   call                RELEASE_E
                    movlw               d'0'                                            ;sposto il cursore sull'offset
                    call                LCD_MOVE_LOC_R3
                    call                DELAY_250ms
SEL_P2_OFFSET_SIN_T btfsc               UP                                              ;UP=1
                    goto                SEL_P2_CONTR                                    ;sposto il cursore sul contrasto
                    btfsc               DOWN                                            ;DOWN=1
                    goto                SEL_P2_OFFSET_SQU                               ;Sposto il cursore sul'offset per l'onda quadra
                    btfsc               LEFT                                            ;LEFT=1
                    goto                MENU_P1                                         ;vado alla terza pagina del menu
                    btfsc               RIGHT                                           ;RIGHT=1
                    goto                MENU_P3                                         ;vado alla terza pagina del menu
                    btfss               ENTER                                           ;ENTER=1 salto
                    goto                SEL_P2_OFFSET_SIN_T                             ;altrimenti ricontrollo i tasti

                    call                DELETE_OK_NEXT                                  ;cancello le scritte BK e NEXT
                    movlw               d'12'                                           ;sposto il cursore sul valore del contrasto
                    call                LCD_MOVE_LOC_R3
                    call                SETUP_OFFSET_SIN                                ;imposto il contrasto
                    call                EXPO_BK_NEXT                                    ;al tirotno espongo di nuovo le scritte BK e NEXT
                    goto                SEL_P2_OFFSET_SIN
;--------------------------------------------------------------------------------------------------------------------------------------------------
;cursore su offset per onda quadra
SEL_P2_OFFSET_SQU   call                RELEASE_E
                    movlw               d'0'                                            ;sposto il cursore sull'offset
                    call                LCD_MOVE_LOC_R4
                    call                DELAY_250ms
SEL_P2_OFFSET_SQU_T btfsc               UP                                              ;UP=1
                    goto                SEL_P2_OFFSET_SIN                               ;sposto il cursore sull'offset per l'onda sinusoidale
                    btfsc               DOWN                                            ;DOWN=1
                    goto                SEL_P2_BACKILL                                  ;Sposto il cursore sulla retro illuminazione
                    btfsc               LEFT                                            ;LEFT=1
                    goto                MENU_P1                                         ;vado alla terza pagina del menu
                    btfsc               RIGHT                                           ;RIGHT=1
                    goto                MENU_P3                                         ;vado alla terza pagina del menu
                    btfss               ENTER                                           ;ENTER=1 salto
                    goto                SEL_P2_OFFSET_SQU_T                             ;altrimenti ricontrollo i tasti

                    call                DELETE_OK_NEXT                                  ;cancello le scritte BK e NEXT
                    movlw               d'12'                                           ;sposto il cursore sul valore del contrasto
                    call                LCD_MOVE_LOC_R4
                    call                SETUP_OFFSET_SQU                                ;imposto il contrasto
                    call                EXPO_BK_NEXT                                    ;al tirotno espongo di nuovo le scritte BK e NEXT
                    goto                SEL_P2_OFFSET_SQU                               ;torno a SEL_P2_CONTR
;
;--------------------------------------------------------------------------------------------------------------------------------------------------
;Gestione spostamento menu PAGINA 3
;--------------------------------------------------------------------------------------------------------------------------------------------------
;esposizione valori iniziali e scritte
MENU_P3             call                RELEASE_T
                    call                LOAD_START_VALUE                                ;carico i valori attualmente salvati in EEPROM
                    call                EXPO_STATIC                                     ;e li espongo
                    call                EXPO_SHAPE
                    call                EXPO_FREQ
                    call                EXPO_PHASE
                    call                EXPO_VPP
                    call                EXPO_BK_SAVE                                    ;espongo le scritte BK e SAVE

;--------------------------------------------------------------------------------------------------------------------------------------------------
;cursore su forma d'onda
SEL_P3_SHAPE        call                RELEASE_E                                       ;Come SEL_P1_xxxx
                    movlw               d'0'
                    call                LCD_MOVE_LOC_R1
                    call                DELAY_250ms
SEL_P3_SHAPE_T      btfsc               UP
                    goto                SEL_P3_VPP
                    btfsc               DOWN
                    goto                SEL_P3_FREQ
                    btfsc               LEFT
                    goto                MENU_P2
                    btfsc               RIGHT
                    goto                SAVE_DATA
                    btfss               ENTER
                    goto                SEL_P3_SHAPE_T

                    call                DELETE_OK_NEXT
                    movlw               d'8'
                    call                LCD_MOVE_LOC_R1
                    call                SETUP_SHAPE
                    call                EXPO_VPP

                    call                EXPO_BK_SAVE
                    goto                SEL_P3_SHAPE
;--------------------------------------------------------------------------------------------------------------------------------------------------
;cursore su frequenza
SEL_P3_FREQ         call                RELEASE_E
                    movlw               d'0'
                    call                LCD_MOVE_LOC_R2
                    call                DELAY_250ms
SEL_P3_FREQ_T       btfsc               UP
                    goto                SEL_P3_SHAPE
                    btfsc               DOWN
                    goto                SEL_P3_PHASE
                    btfsc               LEFT
                    goto                MENU_P2
                    btfsc               RIGHT
                    goto                SAVE_DATA
                    btfss               ENTER
                    goto                SEL_P3_FREQ_T


                    call                EXPO_ARROW
                    movlw               d'15'
                    call                LCD_MOVE_LOC_R2
                    call                SETUP_FREQ
                    call                EXPO_BK_SAVE
                    goto                SEL_P3_FREQ
;--------------------------------------------------------------------------------------------------------------------------------------------------
;cursore su fase
SEL_P3_PHASE        call                RELEASE_E
                    movlw               d'0'
                    call                LCD_MOVE_LOC_R3
                    call                DELAY_250ms
SEL_P3_PHASE_T      btfsc               UP
                    goto                SEL_P3_FREQ
                    btfsc               DOWN
                    goto                SEL_P3_VPP
                    btfsc               LEFT
                    goto                MENU_P2
                    btfsc               RIGHT
                    goto                SAVE_DATA
                    btfss               ENTER
                    goto                SEL_P3_PHASE_T

                    call                DELETE_OK_NEXT
                    movlw               d'8'
                    call                LCD_MOVE_LOC_R3
                    call                SETUP_PHASE
                    call                EXPO_BK_SAVE
                    goto                SEL_P3_PHASE
;--------------------------------------------------------------------------------------------------------------------------------------------------
;cursore su vpp
SEL_P3_VPP          call                RELEASE_E
                    movlw               d'0'
                    call                LCD_MOVE_LOC_R4
                    call                DELAY_250ms
SEL_P3_VPP_T        btfsc               UP
                    goto                SEL_P3_PHASE
                    btfsc               DOWN
                    goto                SEL_P3_SHAPE
                    btfsc               LEFT
                    goto                MENU_P2
                    btfsc               RIGHT
                    goto                SAVE_DATA
                    btfss               ENTER
                    goto                SEL_P3_VPP_T

                    call                DELETE_OK_NEXT
                    movlw               d'9'
                    call                LCD_MOVE_LOC_R4
                    call                SETUP_VPP
                    call                EXPO_BK_SAVE
                    goto                SEL_P3_VPP

;--------------------------------------------------------------------------------------------------------------------------------------------------
;salvataggio dati e ritorno alla pagina 2
SAVE_DATA           call                RELEASE_T                                       ;attendo che tutti i pulsanti vengano rilasciati
                    call                SAVE_START_VALUE                                ;salvo i valori in EEPROM
                    call                EXPO_SAVED                                      ;Espongo il messaggio di salvataggio
                    movlw               TIME_INFO                                       ;attendo
                    call                DELAY_Xms

                    goto                MENU_P2                                         ;torno alla seconda pagina del menu
;--------------------------------------------------------------------------------------------------------------------------------------------------
;uscita dal setup, cancello scritte MENU e tolgo il cursore
MENU_EXIT           call                RELEASE_T
                    movlw               LCD_CURSOR_OFF                                  ;disattivo il cursore
                    call                LCD_SEND_CODE
                    call                DELETE_OK_NEXT                                  ;cancello le scritte OK e NEXT
                    call                EXPO_SWAP                                       ;espongo la scritta SWAP

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       SETUP_SHAPE:
;   Output:         shape
;   Overview:       Imposta la forma d'onda, mantiene il valore di shape tra 0 e 2
;                   shape=0: quadra, shape=1:trian, shape=2: sin
;**************************************************************************************************************************************************
;
SETUP_SHAPE         call                RELEASE_E                                       ;attendo che venga rilasciato il tasto ENTER
                    call                DELAY_250ms                                     ;ritardo
                    call                EXPO_SHAPE                                      ;espongo la forma d'onda selezionata
                    movlw               d'8'                                            ;mantengo il cursore inposizione 8
                    call                LCD_MOVE_LOC_R1

SETUP_SHAPE_T       btfsc               UP                                              ;se UP=1
                    goto                INC_SHAPE                                       ;vado ad incrementare la forma d'onda
                    btfsc               DOWN                                            ;se DOWN=1
                    goto                DEC_SHAPE                                       ;vado a decrementare la forma d'onda
                    btfss               ENTER                                           ;se ENTER=1 salto
                    goto                SETUP_SHAPE_T                                   ;altrimenti ricontrollo i tasti

                    call                RELEASE_E                                       ;attendo che venga rilasciato il tasto ENTER

                    call                SHAPE_TO_CTRL                                   ;imposto dss_ctrl_l a seconda della forma d'onda selezionata
                    clrf                dss_ctrl_h                                      ;cancello dss_ctrl_h
                    call                SPI_SEND_CTRL                                   ;invio al DSS i dati della nuova forma d'onda

                    movff               pot_offset_sin, pot_offset                      ;imposto l'offset per la sinusoide nell'offset da programmare

                    movf                shape,W                                         ;se shape = 0
                    btfsc               STATUS,Z
                    movff               pot_offset_squ, pot_offset                      ;imposto l'offset del'onda quandra nell'offset da programmare altrimenti resta quello della sinusoide

                    call                SPI_SEND_POT_OFFS                               ;programmo il potenziometro digitale per l'offset con il dato contenuto in pot_offset
                                                                                        ;se cambia forma potrebbe cambiare offset
                    return

INC_SHAPE           incf                shape,F                                         ;incremento shape
                    movf                shape,W                                         ;se shape=3
                    xorlw               h'3'
                    btfsc               STATUS,Z
                    decf                shape,F                                         ;riportalo a 2
                    goto                SETUP_SHAPE                                     ;torno a SETUP_SHAPE


DEC_SHAPE           decf                shape,F                                         ;decremento shape
                    btfss               STATUS,C                                        ;se c' riporto
                    clrf                shape                                           ;lo resetto
                    goto                SETUP_SHAPE                                     ;torno a SETUP_SHAPE
;
;**************************************************************************************************************************************************
;   FUNCTION:       SHAPE_TO_CTRL
;   Input:          shape
;   Output:         dss_ctrl_l
;   Overview:       imposta i byte dss_ctrl_l e a seconda della forma d'onda selezionata
;**************************************************************************************************************************************************
;
SHAPE_TO_CTRL       movf                shape,W                                         ;shape=0
                    btfsc               STATUS,Z
                    goto                SET_SQU_CTRL                                    ;imposto i dati per un'onda quadra
                    movf                shape,W                                         ;shape=1
                    xorlw               h'1'
                    btfsc               STATUS,Z
                    goto                SET_TRI_CTRL                                    ;imposto i dati per un'onda triangolare
                    movf                shape,W                                         ;shape=2
                    xorlw               h'2'                                            ;imposto i dati per un'onda sinusoidale
                    btfsc               STATUS,Z
                    goto                SET_SIN_CTRL

                    call                EXPO_ERR_SHAPE                                  ;se nessuno dei valori precedenti espongo l'errore

ERR_LOOP_0          goto                ERR_LOOP_0                                      ;e mi fermo qui (usato in debug, normalemente no dovrebbe verificarsi
                                                                                        ;mai questa condizione)
SET_SQU_CTRL        movlw               b'00101000'
                    movwf               dss_ctrl_l
                    return

SET_TRI_CTRL        movlw               b'00000010'
                    movwf               dss_ctrl_l
                    return

SET_SIN_CTRL        movlw               b'00000000'
                    movwf               dss_ctrl_l
                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       SETUP_FREQ
;   Precondition:   Dichiarare la costanti FREQ_MAX all'inizio del programma
;   Output:         freq_ins_0, freq_ins_1, freq_ins_2, freq_ins_3, freq_ins_4, freq_ins_5, freq_ins_6 (freq_ins_0: LSB)
;   Overview:       imposta il valore della frequenza in 7 variabili BCD
;**************************************************************************************************************************************************
;--------------------------------------------------------------------------------------------------------------------------------------------------
;impostazione freq_ins_0
SETUP_FREQ          call                RELEASE_E                                       ;attendo il rilascio del tasto enter

SETUP_FREQ_0        movlw               FREQ_MAX                                        ;se freq_ins_6 < FREQ_MAX
                    cpfslt              freq_ins_6                                      ;altrimenti se > FREQ_MAX
                    call                SETUP_FREQ_MAX                                  ;vai a SETUP_FREQ_MAX

                    call                DELAY_250ms                                     ;ritardo
                    call                EXPO_FREQ                                       ;espongo la frequenza
                    movlw               d'15'                                           ;mantengo il cursore in posizione 16
                    call                LCD_MOVE_LOC_R2                                 ;della seconda riga (primo valore della frequenza)

SETUP_FREQ_0_T      btfsc               UP                                              ;se UP=1
                    goto                INC_FREQ_0                                      ;vado ad incrementare la variabile
                    btfsc               DOWN                                            ;se DOWN=1
                    goto                DEC_FREQ_0                                      ;vado a decrementare la variabile
                    btfsc               LEFT                                            ;se LEFT=1
                    goto                SETUP_FREQ_1                                    ;passa al valore di sinistra
                    btfss               ENTER                                           ;se ENTER=1 salto e vado a SETUP_PREQ_OUT
                    goto                SETUP_FREQ_0_T                                  ;altrimenti ricontrollo i tasti
                    goto                SETUP_FREQ_OUT

INC_FREQ_0          incf                freq_ins_0,F                                    ;incremento la frequenza
                    movf                freq_ins_0,W                                    ;se freq=10
                    xorlw               h'A'
                    btfsc               STATUS,Z
                    decf                freq_ins_0,F                                    ;lo riporto a 9
                    goto                SETUP_FREQ_0                                    ;torno a SETUP_FREQ_0

DEC_FREQ_0          decf                freq_ins_0,F                                    ;decremento la frequenza
                    btfss               STATUS,C
                    clrf                freq_ins_0                                      ;se c' riporto tiengo a 0 il valore
                    goto                SETUP_FREQ_0                                    ;torno a SETUP_FREQ_0
;--------------------------------------------------------------------------------------------------------------------------------------------------
;impostazione freq_ins_1
SETUP_FREQ_1        movlw               FREQ_MAX                                        ;se freq_ins_6 < FREQ_MAX
                    cpfslt              freq_ins_6                                      ;altrimenti se > FREQ_MAX
                    call                SETUP_FREQ_MAX                                  ;vai a SETUP_FREQ_MAX

                    call                DELAY_250ms                                     ;;ritardo
                    call                EXPO_FREQ                                       ;espongo la frequenza
                    movlw               d'14'                                           ;mantengo il cursore in posizione 15
                    call                LCD_MOVE_LOC_R2

SETUP_FREQ_1_T      btfsc               UP                                              ;se UP=1
                    goto                INC_FREQ_1                                      ;vado ad incrementare la variabile
                    btfsc               DOWN                                            ;se DOWN=1
                    goto                DEC_FREQ_1                                      ;vado a decrementare la variabile
                    btfsc               LEFT                                            ;se LEFT=1
                    goto                SETUP_FREQ_2                                    ;passa al valore di sinistra
                    btfsc               RIGHT                                           ;se RIGHT=1
                    goto                SETUP_FREQ_0                                    ;passa al valore di destra
                    btfss               ENTER                                           ;se ENTER=1 salto e vado a SETUP_PREQ_OUT
                    goto                SETUP_FREQ_1_T                                  ;altrimenti ricontrollo i tasti
                    goto                SETUP_FREQ_OUT

INC_FREQ_1          incf                freq_ins_1,F                                    ;incremento la frequenza
                    movf                freq_ins_1,W                                    ;se freq=10
                    xorlw               h'A'
                    btfsc               STATUS,Z
                    decf                freq_ins_1,F                                    ;riportalo a 9
                    goto                SETUP_FREQ_1                                    ;torno a SETUP_FREQ_1

DEC_FREQ_1          decf                freq_ins_1,F                                    ;decremento la frequenza
                    btfss               STATUS,C
                    clrf                freq_ins_1                                      ;se c' riporto tieni a 0 il valore
                    goto                SETUP_FREQ_1                                    ;torno a SETUP_FREQ_1
;--------------------------------------------------------------------------------------------------------------------------------------------------
;impostazione freq_ins_2
SETUP_FREQ_2        movlw               FREQ_MAX                                        ;se freq_ins_6 < FREQ_MAX
                    cpfslt              freq_ins_6                                      ;altrimenti se > FREQ_MAX
                    call                SETUP_FREQ_MAX                                  ;vai a SETUP_FREQ_MAX

                    call                DELAY_250ms                                     ;ritardo
                    call                EXPO_FREQ                                       ;espongo la frequenza
                    movlw               d'13'                                           ;mantengo il cursore in posizione 14
                    call                LCD_MOVE_LOC_R2

SETUP_FREQ_2_T      btfsc               UP                                              ;se UP=1
                    goto                INC_FREQ_2                                      ;vado ad incrementare la variabile
                    btfsc               DOWN                                            ;se DOWN=1
                    goto                DEC_FREQ_2                                      ;vado a decrementare la variabile
                    btfsc               LEFT                                            ;se LEFT=1
                    goto                SETUP_FREQ_3                                    ;passa al valore di sinistra
                    btfsc               RIGHT                                           ;se RIGHT=1
                    goto                SETUP_FREQ_1                                    ;passa al valore di destra
                    btfss               ENTER                                           ;se ENTER=1 salto e vado a SETUP_PREQ_OUT
                    goto                SETUP_FREQ_2_T                                  ;altrimenti ricontrollo i tasti
                    goto                SETUP_FREQ_OUT

INC_FREQ_2          incf                freq_ins_2,F                                    ;incremento la frequenza
                    movf                freq_ins_2,W                                    ;se freq=10
                    xorlw               h'A'
                    btfsc               STATUS,Z
                    decf                freq_ins_2,F                                    ;riportalo a 9
                    goto                SETUP_FREQ_2                                    ;torno a SETUP_FREQ_2

DEC_FREQ_2          decf                freq_ins_2,F                                    ;decremento la frequenza
                    btfss               STATUS,C
                    clrf                freq_ins_2                                      ;se c' riporto tieni a 0 il valore
                    goto                SETUP_FREQ_2                                    ;torno a SETUP_FREQ_2
;--------------------------------------------------------------------------------------------------------------------------------------------------
;impostazione freq_ins_3
SETUP_FREQ_3        movlw               FREQ_MAX                                        ;se freq_ins_6 < FREQ_MAX
                    cpfslt              freq_ins_6                                      ;altrimenti se > FREQ_MAX
                    call                SETUP_FREQ_MAX                                  ;vai a SETUP_FREQ_MAX

                    call                DELAY_250ms                                     ;ritardo
                    call                EXPO_FREQ                                       ;espongo la frequenza
                    movlw               d'11'                                           ;mantengo il cursore in posizione 12
                    call                LCD_MOVE_LOC_R2

SETUP_FREQ_3_T      btfsc               UP                                              ;se UP=1
                    goto                INC_FREQ_3                                      ;vado ad incrementare la variabile
                    btfsc               DOWN                                            ;se DOWN=1
                    goto                DEC_FREQ_3                                      ;vado a decrementare la variabile
                    btfsc               LEFT                                            ;se LEFT=1
                    goto                SETUP_FREQ_4                                    ;passa al valore di sinistra
                    btfsc               RIGHT                                           ;se RIGHT=1
                    goto                SETUP_FREQ_2                                    ;passa al valore di destra
                    btfss               ENTER                                           ;se ENTER=1 salto e vado a SETUP_PREQ_OUT
                    goto                SETUP_FREQ_3_T                                  ;altrimenti ricontrollo i tasti
                    goto                SETUP_FREQ_OUT

INC_FREQ_3          incf                freq_ins_3,F                                    ;incremento la frequenza
                    movf                freq_ins_3,W                                    ;se freq=10
                    xorlw               h'A'
                    btfsc               STATUS,Z
                    decf                freq_ins_3,F                                    ;riportalo a 9
                    goto                SETUP_FREQ_3                                    ;torno a SETUP_FREQ_3

DEC_FREQ_3          decf                freq_ins_3,F                                    ;decremento la frequenza
                    btfss               STATUS,C
                    clrf                freq_ins_3                                      ;se c' riporto tieni a 0 il valore
                    goto                SETUP_FREQ_3                                    ;torno a SETUP_FREQ_3
;--------------------------------------------------------------------------------------------------------------------------------------------------
;impostazione freq_ins_4
SETUP_FREQ_4        movlw               FREQ_MAX                                        ;se freq_ins_6 < FREQ_MAX
                    cpfslt              freq_ins_6                                      ;altrimenti se > FREQ_MAX
                    call                SETUP_FREQ_MAX                                  ;vai a SETUP_FREQ_MAX

                    call                DELAY_250ms                                     ;ritardo
                    call                EXPO_FREQ                                       ;espongo la frequenza
                    movlw               d'10'                                           ;mantengo il cursore in posizione 11
                    call                LCD_MOVE_LOC_R2

SETUP_FREQ_4_T      btfsc               UP                                              ;se UP=1
                    goto                INC_FREQ_4                                      ;vado ad incrementare la variabile
                    btfsc               DOWN                                            ;se DOWN=1
                    goto                DEC_FREQ_4                                      ;vado a decrementare la variabile
                    btfsc               LEFT                                            ;se LEFT=1
                    goto                SETUP_FREQ_5                                    ;passa al valore di sinistra
                    btfsc               RIGHT                                           ;se RIGHT=1
                    goto                SETUP_FREQ_3                                    ;passa al valore di destra
                    btfss               ENTER                                           ;se ENTER=1 salto e vado a SETUP_PREQ_OUT
                    goto                SETUP_FREQ_4_T                                  ;altrimenti ricontrollo i tasti
                    goto                SETUP_FREQ_OUT

INC_FREQ_4          incf                freq_ins_4,F                                    ;incremento la frequenza
                    movf                freq_ins_4,W                                    ;se freq=10
                    xorlw               h'A'                                            ;torno a SETUP_FREQ_4
                    btfsc               STATUS,Z
                    decf                freq_ins_4,F                                    ;riportalo a 9
                    goto                SETUP_FREQ_4

DEC_FREQ_4          decf                freq_ins_4,F                                    ;decremento la frequenza
                    btfss               STATUS,C
                    clrf                freq_ins_4                                      ;se c' riporto tieni a 0 il valore
                    goto                SETUP_FREQ_4                                    ;torno a SETUP_FREQ_4
;--------------------------------------------------------------------------------------------------------------------------------------------------
;impostazione freq_ins_5
SETUP_FREQ_5        movlw               FREQ_MAX                                        ;se freq_ins_6 < FREQ_MAX
                    cpfslt              freq_ins_6                                      ;altrimenti se > FREQ_MAX
                    call                SETUP_FREQ_MAX                                  ;vai a SETUP_FREQ_MAX

                    call                DELAY_250ms                                     ;ritardo
                    call                EXPO_FREQ                                       ;espongo la frequenza
                    movlw               d'09'                                           ;mantengo il cursore in posizione 10
                    call                LCD_MOVE_LOC_R2

SETUP_FREQ_5_T      btfsc               UP                                              ;se UP=1
                    goto                INC_FREQ_5                                      ;vado ad incrementare la variabile
                    btfsc               DOWN                                            ;se DOWN=1
                    goto                DEC_FREQ_5                                      ;vado a decrementare la variabile
                    btfsc               LEFT                                            ;se LEFT=1
                    goto                SETUP_FREQ_6                                    ;passa al valore di sinistra
                    btfsc               RIGHT                                           ;se RIGHT=1
                    goto                SETUP_FREQ_4                                    ;passa al valore di destra
                    btfss               ENTER                                           ;se ENTER=1 salto e vado a SETUP_PREQ_OUT
                    goto                SETUP_FREQ_5_T                                  ;altrimenti ricontrollo i tasti
                    goto                SETUP_FREQ_OUT

INC_FREQ_5          incf                freq_ins_5,F                                    ;incremento la frequenza
                    movf                freq_ins_5,W                                    ;se freq=10
                    xorlw               h'A'
                    btfsc               STATUS,Z
                    decf                freq_ins_5,F                                    ;riportalo a 9
                    goto                SETUP_FREQ_5                                    ;torno a SETUP_FREQ_5

DEC_FREQ_5          decf                freq_ins_5,F                                    ;decremento la frequenza
                    btfss               STATUS,C
                    clrf                freq_ins_5                                      ;se c' riporto tieni a 0 il valore
                    goto                SETUP_FREQ_5                                    ;torno a SETUP_FREQ_5
;--------------------------------------------------------------------------------------------------------------------------------------------------
;impostazione freq_ins_6
SETUP_FREQ_6        movlw               FREQ_MAX                                        ;se freq_ins_6 < FREQ_MAX
                    cpfslt              freq_ins_6                                      ;altrimenti se > FREQ_MAX
                    call                SETUP_FREQ_MAX                                  ;vai a SETUP_FREQ_MAX

                    call                DELAY_250ms                                     ;ritardo
                    call                EXPO_FREQ                                       ;espongo la frequenza
                    movlw               d'07'                                           ;mantengo il cursore in posizione 8
                    call                LCD_MOVE_LOC_R2

SETUP_FREQ_6_T      btfsc               UP                                              ;se UP=1
                    goto                INC_FREQ_6                                      ;vado ad incrementare la variabile
                    btfsc               DOWN                                            ;se DOWN=1
                    goto                DEC_FREQ_6                                      ;vado a decrementare la variabile
                    btfsc               RIGHT                                           ;se RIGHT=1
                    goto                SETUP_FREQ_5                                    ;passa al valore di destra
                    btfss               ENTER                                           ;se ENTER=1 salto e vato a SETUP_PREQ_OUT
                    goto                SETUP_FREQ_6_T                                  ;altrimenti ricontrollo i tasti
                    goto                SETUP_FREQ_OUT

INC_FREQ_6          incf                freq_ins_6,F                                    ;incremento la frequenza
                    movf                freq_ins_6,W                                    ;se freq=10
                    xorlw               h'A'
                    btfsc               STATUS,Z
                    decf                freq_ins_6,F                                    ;riportalo a 9
                    goto                SETUP_FREQ_6                                    ;torno a SETUP_FREQ_6

DEC_FREQ_6          decf                freq_ins_6,F                                    ;decremento la frequenza
                    btfss               STATUS,C
                    clrf                freq_ins_6                                      ;se c' riporto tieni a 0 il valore
                    goto                SETUP_FREQ_6                                    ;torno a SETUP_FREQ_6
;--------------------------------------------------------------------------------------------------------------------------------------------------
;Uscita dai cicli di setup della frequenza
SETUP_FREQ_OUT      call                RELEASE_T

                    call				RESET_VAR_CALC                                  ;reset variabili moltiplicazione
                    call				MOLTIP                                          ;moltiplico la frequenza inserita per M
 					call				BCD9_TO_HEX4                                    ;converto il risultato in 4 byte esadecimali

                    movlw               b'00100000'                                     ;mando al dss il CONTROL REGISTER per scrivere una frequenza completa
                    movwf               dss_ctrl_h                                      ;D15-D14:00 (control register) ;D13:1 (2 operazioni di scrittura)
                    call                SPI_SEND_CTRL                                   ;e imposto la forma d'onda (dss_ctrl_l impostato in SHAPE_TO_CTRL)

                    call                SPI_SEND_FREQ                                   ;mando i dati relativi alla nuova frequenza al DSS

                    return
;--------------------------------------------------------------------------------------------------------------------------------------------------
;routine per tenere il valore massimo a 8.000.000
SETUP_FREQ_MAX      movlw               FREQ_MAX
                    movwf               freq_ins_6
                    clrf                freq_ins_5
                    clrf                freq_ins_4
                    clrf                freq_ins_3
                    clrf                freq_ins_2
                    clrf                freq_ins_1
                    clrf                freq_ins_0
                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       SETUP_PHASE
;   Output:         phase_h
;   Overview:       incrementa o decrementa il valore del byte alto da mandare al DSS per l'impostazione della fase.
;                   valori possibili: 0,2,4,6,8,A,C,E (incrementi di 45)
;**************************************************************************************************************************************************
;
SETUP_PHASE         call                RELEASE_E
                    call                DELAY_250ms                                     ;ritardo
                    call                EXPO_PHASE                                      ;espongo il valore attuale della fase
                    movlw               d'8'                                            ;mantengo il cursore in posizione 9
                    call                LCD_MOVE_LOC_R3                                 ;della terza riga

SETUP_PHASE_T       btfsc               UP                                              ;UP=1
                    goto                INC_PHASE                                       ;vado ad incrementare la fase
                    btfsc               DOWN                                            ;DOWN=1
                    goto                DEC_PHASE                                       ;vado a decrementare la fase
                    btfss               ENTER                                           ;ENTER=1 salto
                    goto                SETUP_PHASE_T                                   ;altrimenti ricontrollo i tasti

                    call                RELEASE_E                                       ;attendo che venga rilasciato il tasto ENTER

                    call                SPI_SEND_PHASE                                  ;mando la nuova fase al DSS
                    return                                                              ;e ritorno

INC_PHASE           movf                phase_h,W                                       ;incremento il valore della fase a step di  2
                    addlw               h'2'                                            ;e lo tengo tra 0 e 0E
                    movwf               phase_h
                    xorlw               h'10'
                    btfss               STATUS,Z
                    goto                SETUP_PHASE

                    movlw               h'0E'
                    movwf               phase_h
                    goto                SETUP_PHASE

DEC_PHASE           movlw               h'2'                                            ;decremento il valore della fase a step di 2 e
                    subwf               phase_h
                    btfss               STATUS,C                                        ;se c' riporto
                    clrf                phase_h                                         ;lo riporto a 0
                    goto                SETUP_PHASE
;
;**************************************************************************************************************************************************
;   FUNCTION:       SETUP_VPP
;   Output:         vpp
;   Overview:       incrementa o decrementa il valore del vpp mantenedolo tra 1 e 8
;**************************************************************************************************************************************************
;--------------------------------------------------------------------------------------------------------------------------------------------------
SETUP_VPP           call                RELEASE_E                                       ;attendo che venga rilasciato il tasto ENTER
                    call                DELAY_250ms                                     ;ritardo
                    call                EXPO_VPP                                        ;espongo vpp
                    movlw               d'9'                                            ;mantengo il cursore inposizione 9
                    call                LCD_MOVE_LOC_R4

SETUP_VPP_T         btfsc               UP                                              ;se UP=1
                    goto                INC_VPP                                         ;vado ad incrementare vpp
                    btfsc               DOWN                                            ;se DOWN=1
                    goto                DEC_VPP                                         ;vado a decrementare vpp
                    btfss               ENTER                                           ;se ENTER=1 salto
                    goto                SETUP_VPP_T                                     ;altrimenti ricontrollo i tasti

                    call                RELEASE_E
                    call                VPP_TO_MUX                                      ;ricavo il valore dallo switch da abilitare
                    call                SPI_SEND_MUX_VPP                                ;programmo imultiplexer con il valore ottenuto
                    return

INC_VPP             incf                vpp,F                                           ;incremento shape
                    movf                vpp,W                                           ;se vpp=9
                    xorlw               h'9'
                    btfsc               STATUS,Z
                    decf                vpp,F                                           ;riportalo a 8
                    goto                SETUP_VPP                                       ;torno a SETUP_VPP
  
DEC_VPP             decf                vpp,F                                           ;decremento shape
                    btfsc               STATUS,Z                                        ;se  0
                    incf                vpp                                             ;lo riporto a 1
                    goto                SETUP_VPP                                       ;torno a SETUP_VPP;
;
;**************************************************************************************************************************************************
;   FUNCTION:       VPP_TO_MUX
;   Input:          vpp
;   Output:         mux_vpp
;   Overview:       Shifta a sinistra per n volte quante i 4 bit piu bassi di vpp se il bit 4 di vpp = 1 disabilita il rele, se = 0 abilita il rele
;**************************************************************************************************************************************************
;
VPP_TO_MUX          movf                vpp,W
                    andlw               b'00001111'                                     ;maschero i 4 bit piu bassi
                    movwf               count
                    movlw               b'10000000'                                     ;imposto valore base per il ciclo di shift
                    movwf               mux_vpp

MUX_LOOP            rlncf               mux_vpp                                         ;ruoto a sinistra di n volte n=4bit bassi di vpp
                    decfsz              count
                    goto                MUX_LOOP

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       SETUP_BACKILL
;   Output:         CCPR2L
;   Overview:       Incremeta o decrementa il valore del dutycicle relativo alla retroilluminazione e lo salva nella EEPROM
;                   duty con valore variabile da 0 a FA con incrementi di 19
;**************************************************************************************************************************************************
;
SETUP_BACKILL       call                RELEASE_E
                    call                DELAY_250ms                                     ;debounce
                    call                EXPO_BACKILL                                    ;esponi valore
                    movlw               d'12'                                           ;mantengo il cursore in posizione 12
                    call                LCD_MOVE_LOC_R1

SETUP_BACKILL_T     btfsc               UP                                              ;Se UP=1
                    goto                INC_BACKILL                                     ;vai and incrementare il valore
                    btfsc               DOWN                                            ;se DOWN=1
                    goto                DEC_BACKILL                                     ;vai a decrementare il valore
                    btfss               ENTER                                           ;se ENTER=1 salta e salva il valore
                    goto                SETUP_BACKILL_T                                 ;altrimenti resta a SETUP_BACKILL

                    call                RELEASE_E                                       ;attendo che venga rilasciato il tasto ENTER

                    movlw               EE_duty_backill                                 ;salvo il valore del Duti cicle relativo alla retroilluminazione
                    movwf               ee_adr                                          ;in EEPROM
                    movff               CCPR2L, ee_data
                    call                WRITE_EE
                    return

INC_BACKILL         movf                CCPR2L,W                                        ;incremento il valore del dutycicle a step di '19' in modo da avere 10 step da 0 a FA
                    addlw               h'19'                                           ;se la somma provoca riposto mantieni h'FA' come valore massimo
                    btfsc               STATUS,C
                    movlw               h'FA'
                    movwf               CCPR2L
                    goto                SETUP_BACKILL

DEC_BACKILL         movlw               h'19'                                           ;decremento il valore del dutycicle
                    subwf               CCPR2L,W
                    btfss               STATUS,C                                        ;se la sotrazione va in negativo
                    movlw               d'0'                                            ;mantengo il valore a 0
                    movwf               CCPR2L
                    goto                SETUP_BACKILL
;
;**************************************************************************************************************************************************
;   FUNCTION:       SETUP_CONTR
;   Output:         CCPR1L
;   Overview:       incremeta o decrementa il valore del dutycicle relativo alcontrasto e lo salva nella EEPROM
;                   duty con valore variabile da 0 a A0 con incrementi di 10
;**************************************************************************************************************************************************
;
SETUP_CONTR         call                RELEASE_E
                    call                EXPO_CONTR                                      ;esponi valore
                    movlw               d'12'                                           ;mantengo il cursore in posizione 12
                    call                LCD_MOVE_LOC_R2
                    call                DELAY_250ms                                     ;debounce

SETUP_CONTR_T       btfsc               UP                                              ;UP decremento (logica inversa duty)
                    goto                DEC_CONTR
                    btfsc               DOWN                                            ;DOWN incremento
                    goto                INC_CONTR
                    btfss               ENTER                                           ;se ENTER=1 salta e salva il valore
                    goto                SETUP_CONTR_T

                    call                RELEASE_E                                       ;attendo che venga rilasciato il tasto ENTER

                    movlw               EE_duty_contrast                                ;salvo il dato relativo al duty del contrasto
                    movwf               ee_adr                                          ;in EEPROM
                    movff               CCPR1L, ee_data
                    call                WRITE_EE
                    return

INC_CONTR           movf                CCPR1L,W                                        ;oncremento il valore del dutycicle a step di 10 in modo da avere 10 step da 0 ad A0
                    addlw               h'10'
                    btfsc               STATUS,C
                    movlw               h'A0'                                           ;Se supero A0 mantengo valore massimo A0
                    movwf               CCPR1L
                    goto                SETUP_CONTR

DEC_CONTR           movlw               h'10'                                           ;decremneto dutycicle
                    subwf               CCPR1L,W
                    btfss               STATUS,C
                    movlw               d'0'                                            ;mantengo il valore a 0
                    movwf               CCPR1L
                    goto                SETUP_CONTR
;
;**************************************************************************************************************************************************
;   FUNCTION:       SETUP_OFFSET
;   Output:         pot_offset_sin
;   Overview:       incremeta o decrementa il valore dell'offset e salva il dato in EEPROM
;**************************************************************************************************************************************************
;
SETUP_OFFSET_SIN    call                RELEASE_E
                    call                EXPO_OFFSET_SIN                                 ;esponi valore
                    movlw               d'12'                                           ;mantengo il cursore in posizione 12
                    call                LCD_MOVE_LOC_R3
                    movff               pot_offset_sin, pot_offset                      ;imposto nella variabile di programmazione il valore di pot_offset_sin
                    call                SPI_SEND_POT_OFFS                               ;programmo l'offset
                    call                DELAY_100ms                                     ;ritardo

SETUP_OFFSET_T      btfsc               UP                                              ;UP = 1
                    goto                INC_OFFSET
                    btfsc               DOWN                                            ;DOWN incremento
                    goto                DEC_OFFSET
                    btfss               ENTER                                           ;se ENTER=1 salta e salva il valore
                    goto                SETUP_OFFSET_T

                    call                RELEASE_E                                       ;attendo che venga rilasciato il tasto ENTER

                    movlw               EE_pot_offset_sin                               ;salvo il dato relativo all'offset_sin
                    movwf               ee_adr                                          ;in EEPROM
                    movff               pot_offset_sin, ee_data
                    call                WRITE_EE
                    return

INC_OFFSET          incf                pot_offset_sin,F                                ;incremento offset
                    btfsc               STATUS,C                                        ;se pot_offset_sin > h'FF'
                    decf                pot_offset_sin,F                                ;decrementalo di 1 (mantengo a FF massimo)
                    goto                SETUP_OFFSET_SIN

DEC_OFFSET          decf                pot_offset_sin,F                                ;decremento offset
                    btfss               STATUS,C                                        ;se pot_offset_sin < 0
                    incf                pot_offset_sin,F                                ;incremento di 1 (mantengo a 0 minimo)
                    goto                SETUP_OFFSET_SIN
;
;**************************************************************************************************************************************************
;   FUNCTION:       SETUP_OFFSET_SQU
;   Output:         pot_offset_squ
;   Overview:       incremeta o decrementa il valore dell'offset e salva il dato in EEPROM
;**************************************************************************************************************************************************
;
SETUP_OFFSET_SQU    call                RELEASE_E
                    call                EXPO_OFFSET_SQU                                 ;esponi valore
                    movlw               d'12'                                           ;mantengo il cursore in posizione 12
                    call                LCD_MOVE_LOC_R4
                    movff               pot_offset_squ, pot_offset                      ;imposto nella variabile di programmazione il valore di pot_offset_squ
                    call                SPI_SEND_POT_OFFS                               ;programmo l'offset
                    call                DELAY_100ms                                     ;debounce

SETUP_OFFSET_T_SQU  btfsc               UP                                              ;UP = 1
                    goto                INC_OFFSET_SQU
                    btfsc               DOWN                                            ;DOWN incremento
                    goto                DEC_OFFSET_SQU
                    btfss               ENTER                                           ;se ENTER=1 salta e salva il valore
                    goto                SETUP_OFFSET_T_SQU

                    call                RELEASE_E                                       ;attendo che venga rilasciato il tasto ENTER

                    movlw               EE_pot_offset_squ                               ;salvo il dato relativo all'offset_squ
                    movwf               ee_adr                                          ;in EEPROM
                    movff               pot_offset_squ, ee_data
                    call                WRITE_EE
                    return

INC_OFFSET_SQU      incf                pot_offset_squ,F                                ;incremento offset
                    btfsc               STATUS,C
                    decf                pot_offset_squ,F
                    goto                SETUP_OFFSET_SQU

DEC_OFFSET_SQU      decf                pot_offset_squ,F                                ;decremento offset
                    btfss               STATUS,C
                    incf                pot_offset_squ,F
                    goto                SETUP_OFFSET_SQU


;
;**************************************************************************************************************************************************
;   FUNCTION:       MENU_SWAP
;   Overview:       Menu e gestione Swap freq da 1Hz a 1Mhz in step ampi
;**************************************************************************************************************************************************
;
MENU_SWAP           call                RELEASE_T
                    call                EXPO_MENU_SWAP                                  ;Espongo il menu di SWAP

                    movlw               LCD_CURSOR_ON                                   ;e attivo la linea cursore
                    call                LCD_SEND_CODE
;--------------------------------------------------------------------------------------------------------------------------------------------------
;swap da 1Hz a 1MHz
SWAP_1_TO_1M        call                RELEASE_T
                    movlw               d'0'                                            ;imposto il cursore alla prima riga
                    call                LCD_MOVE_LOC_R1
                    btfsc               RIGHT                                           ;RIGHT = 1
                    goto                SWAP_1_TO_1M_START                              ;VADO a SWAP_1_TO_1M_START
                    btfsc               LEFT                                            ;LEFT = 1
                    goto                SWAP_OUT                                        ;vado a SWAP_OUT
                    btfsc               UP                                              ;UP = 1
                    goto                SWAP_20_TO_48K                                  ;vado a SWAP_20_TO_48K
                    btfsc               DOWN
                    goto                SWAP_20_TO_48K
                    goto                SWAP_1_TO_1M                                    ;altrimenti ricontrollo i tasti

SWAP_1_TO_1M_START  call                RELEASE_T
                    movlw               LCD_CURSOR_OFF                                  ;disabilito il cursore
                    call                LCD_SEND_CODE
                    call                EXPO_STOP                                       ;Espongo la scritta STOP
                    movlw               h'08'                                           ;imposto il valore iniziale della frequenza
                    movwf               freq_hex_s_0
                    clrf                freq_hex_s_1
                    clrf                freq_hex_s_2
                    clrf                freq_hex_s_3

SWAP_1_TO_1M_LOOP   btfsc               RIGHT                                           ;RIGHT = 1
                    goto                MENU_SWAP                                       ;fermo lo swap torno al MENU_SWAP

                    movff               freq_hex_s_0, freq_hex_0                        ;sposto i valori della frequenza di swap nelle variabili standard
                    movff               freq_hex_s_1, freq_hex_1
                    movff               freq_hex_s_2, freq_hex_2
                    movff               freq_hex_s_3, freq_hex_3

                    movlw               b'00100000'                                     ;mando al dss il CONTROL REGISTER per scrivere una frequenza completa
                    movwf               dss_ctrl_h                                      ;D15-D14:00 (control register) ;D13:1 (2 operazioni di scrittura)
                    call                SPI_SEND_CTRL                                   ;e imposto la forma d'onda (dss_ctrl_l impostato in SHAPE_TO_CTRL)
                    call                SPI_SEND_FREQ                                   ;mando i dati relativi alla nuova frequenza al DSS

                    call                DELAY_180us                                     ;ritardo
					call                DELAY_180us

                    incf                freq_hex_s_1,F                                  ;incremento il secondo byte della frequenza di swap (altrimenti sarebbe lunghissimo)
                    btfss               STATUS,C                                        ;se arriva a 0 salta
                    goto                SWAP_1_TO_1M_LOOP
                    incf                freq_hex_s_2,F                                  ;e icrementa il terzo byte

                    movf                freq_hex_s_2,W                                  ;se il terzo byte arricva a 7D (1Mhz @32,768MHz)
                    xorlw               h'7D'
                    btfss               STATUS,Z
                    goto                SWAP_1_TO_1M_LOOP
                    goto                MENU_SWAP                                       ;torno a MENU_SWAP
;--------------------------------------------------------------------------------------------------------------------------------------------------
;swap da 20Hz a 48KHz
SWAP_20_TO_48K      call                RELEASE_T
                    movlw               d'0'                                            ;imposto il cursore slla seconda riga
                    call                LCD_MOVE_LOC_R2
                    btfsc               RIGHT                                           ;RIGHT = 1
                    goto                SWAP_20_TO_48K_START                            ;vado a SWAP_20_TO_48K_START
                    btfsc               LEFT                                            ;LEFT = 1
                    goto                SWAP_OUT                                        ;vado a SWAP_OUT
                    btfsc               UP                                              ;UP = 1
                    goto                SWAP_1_TO_1M                                    ;vado a SWAP_1TO_1M
                    btfsc               DOWN
                    goto                SWAP_1_TO_1M
                    goto                SWAP_20_TO_48K                                  ;altrimenti ricontrollo i tasti

SWAP_20_TO_48K_START
                    call                RELEASE_T
                    movlw               LCD_CURSOR_OFF                                  ;disabilito il cursore
                    call                LCD_SEND_CODE
                    call                EXPO_STOP                                       ;Espongo la scritta STOP
                    movlw               h'A3'                                           ;imposto il valore iniziale della frequenza
                    movwf               freq_hex_s_0
                    clrf                freq_hex_s_1
                    clrf                freq_hex_s_2
                    clrf                freq_hex_s_3

SWAP_20_TO_48K_LOOP btfsc               RIGHT                                           ;RIGHT = 1
                    goto                MENU_SWAP                                       ;torno a MENU_SWAP

                    movff               freq_hex_s_0, freq_hex_0                        ;metto i valori della frequenza di swap nelle variabili standard
                    movff               freq_hex_s_1, freq_hex_1
                    movff               freq_hex_s_2, freq_hex_2
                    movff               freq_hex_s_3, freq_hex_3

                    movlw               b'00100000'                                     ;mando al dss il CONTROL REGISTER per scrivere una frequenza completa
                    movwf               dss_ctrl_h                                      ;D15-D14:00 (control register) ;D13:1 (2 operazioni di scrittura)
                    call                SPI_SEND_CTRL                                   ;e imposto la forma d'onda (dss_ctrl_l impostato in SHAPE_TO_CTRL)
                    call                SPI_SEND_FREQ                                   ;mando i dati relativi alla nuova frequenza al DSS

                    call                DELAY_180us                                     ;ritardo

                    incf                freq_hex_s_0,F                                  ;incremento il primo byte
                    btfss               STATUS,C                                        ;se arriva a 0 salta
                    goto                SWAP_20_TO_48K_LOOP
                    incf                freq_hex_s_1,F                                  ;e icremento il secondo byte

                    btfss               STATUS,C                                        ;se arriva a 0 salta
                    goto                SWAP_20_TO_48K_LOOP
                    incf                freq_hex_s_2,F                                  ;e incremento il terzo byte

                    movf                freq_hex_s_2,W                                  ;quando il terzo byte arriva a 6
                    xorlw               h'06'
                    btfss               STATUS,Z
                    goto                SWAP_20_TO_48K_LOOP
                    goto                MENU_SWAP                                       ;torno a MENU_SWAP

SWAP_OUT            call                RELEASE_T
                    movlw               LCD_CURSOR_OFF                                  ;disabilito il cursore
                    call                LCD_SEND_CODE
                    call                EXPO_STATIC                                     ;Espongo le scritte statiche su display
                    call                EXPO_SHAPE                                      ;Espongo la forma d'onda
                    call                EXPO_FREQ                                       ;Espongo la frequenza
                    call                EXPO_PHASE                                      ;Espongo la fase
                    call                EXPO_VPP
                    call                EXPO_SWAP
                                                                                        ;prima di tornare reimposto i valori della frequenza con quella impostata a menu
                    call				RESET_VAR_CALC                                  ;reset variabili moltiplicazione
                    call				MOLTIP                                          ;moltiplico la frequenza caricata per M
 					call				BCD9_TO_HEX4                                    ;converto il risultato in 4 byte esadecimali

                    movlw               b'00100000'                                     ;imposto dss_ctrl_h per mandare una frequenza completa (4byte)
                    movwf               dss_ctrl_h
                    call                SPI_SEND_CTRL                                   ;mando forma d'onda e avviso di programmazione frequenza  (dss_ctrl_h e dss_ctrl_l al DSS)

                    call                SPI_SEND_FREQ                                   ;mando i dati relativi alla frequenza al DSS


                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       RELEASE_E
;   Overview:       Attende fin che il tasto ENTER non viene rilasciato
;**************************************************************************************************************************************************
;
RELEASE_E           btfsc               ENTER                                           ;resto qui fin che non viene rilasciato il tasto enter
                    goto                RELEASE_E
                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       RELEASE_T
;   Overview:       Attende fin che tutti i tasti non vengono rilasciati
;**************************************************************************************************************************************************
;
RELEASE_T           btfsc				ENTER
					goto				RELEASE_T
					btfsc				LEFT
					goto				RELEASE_T
					btfsc				RIGHT
					goto				RELEASE_T
					btfsc				UP
					goto				RELEASE_T
					btfsc				DOWN
					goto				RELEASE_T

                	return
;
;**************************************************************************************************************************************************
;   FUNCTION:       PWM_START
;   Precondition:   Aver caricato in CCPR1L e CCPR2L i rispettivi valori tramite lettura EEPROM o primo avvio
;   Overview:       Avvia i 2 moduli PWM per il controllo del contrasto e della retroilluminazion
;**************************************************************************************************************************************************
;
PWM_START           bcf                 T2CON,0                                         ;Prescaler T2 = 1
                    bcf                 T2CON,1
                    movlw               h'FF'
                    movwf               PR2                                             ;imposto il periodo

					bsf                 T2CON,2                                         ;attivo il timer 2
                    movlw               b'00001100'                                     ;Attivo PWM1 (contrasto)
                    movwf               CCP1CON
                    movlw               b'00001100'                                     ;attivo PWM2 (retroilluminazione)
                    movwf               CCP2CON

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       FIRST_START
;   Output:         shape=2, freq_ins_3=1, pot_offset=80, vpp=1, CCPR1L=30, CCPR2L=FA
;   Overview:       Imposta dei dati di base al primo avvio e li salva in EEPROM.
;                   Imposta a 1 il flag in EEPROM di primo caricamento avvenuto (0x0E)
;**************************************************************************************************************************************************
;
FIRST_START         movlw               h'02'                                           ;imposto i valori iniziali
                    movwf               shape
                    movlw               h'01'
                    movwf               freq_ins_3

                    movlw               h'30'                                           ;valori standard PWM contrasto e retroilluminazione
                    movwf               CCPR1L
                    movlw               h'FA'
                    movwf               CCPR2L

                    movlw               h'80'                                           ;imposto l'offset a met scala
                    movwf               pot_offset_sin
                    movlw               h'A0'
                    movwf               pot_offset_squ

                    clrf                freq_ins_0
                    clrf                freq_ins_1
                    clrf                freq_ins_2
                    clrf                freq_ins_4
                    clrf                freq_ins_5
                    clrf                freq_ins_6
                    clrf                phase_h

                    movlw               d'1'                                            ;vpp = 1
                    movwf               vpp

                    call                SAVE_START_VALUE                                ;salvo i valori iniziali in EEPROM

                    movlw               EE_duty_contrast                                ;salvo il valore del Duti cicle relativo al contrasto
                    movwf               ee_adr                                          ;in EEPROM
                    movff               CCPR1L, ee_data
                    call                WRITE_EE

                    movlw               EE_duty_backill                                 ;salvo il valore del Duti cicle relativo alla retroilluminazione
                    movwf               ee_adr                                          ;in EEPROM
                    movff               CCPR2L, ee_data
                    call                WRITE_EE

                    movlw               EE_pot_offset_sin                               ;salvo i valori relativi agli offset standard in EEPROM
                    movwf               ee_adr
                    movff               pot_offset_sin, ee_data
                    call                WRITE_EE

                    movlw               EE_pot_offset_squ
                    movwf               ee_adr
                    movff               pot_offset_squ, ee_data
                    call                WRITE_EE

                    movlw               EE_first_start                                  ;setto il flag di primo avvio avvenuto in EEPROM
                    movwf               ee_adr
                    movlw               h'01'
                    movwf               ee_data
                    call                WRITE_EE

                    call                EXPO_FIRST_START                                ;espongo scritta "inizializzazione"
                    movlw               TIME_INFO
                    call                DELAY_Xms

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       LOAD_START_VALUE
;   Output:         CCPR1L, CCPR2L, shape, freq_ins_0, freq_ins_1, freq_ins_2, freq_ins_3, freq_ins_4, freq_ins_5, freq_ins_6, phase_h, vpp, pot_offset_sin, pot_offset_squ
;   Overview:       Carica i valori salvati in EEPROM
;**************************************************************************************************************************************************
;
LOAD_START_VALUE    clrf                ee_adr

                    call                READ_EE
                    movff               ee_data, CCPR1L                                 ;carico il valore salvato del dutycicle per il contrasto
                    incf                ee_adr,F
                    call                READ_EE                                         ;carico il valore salvato del dutycicle per la retroilluminazione
                    movff               ee_data, CCPR2L
                    incf                ee_adr,F                                        ;leggo tutti i dati in EEPROM e li salvo nelle variabili
                    call                READ_EE
                    movff               ee_data, shape
                    incf                ee_adr,F
                    call                READ_EE
                    movff               ee_data, freq_ins_0
                    incf                ee_adr,F
                    call                READ_EE
                    movff               ee_data, freq_ins_1
                    incf                ee_adr,F
                    call                READ_EE
                    movff               ee_data, freq_ins_2
                    incf                ee_adr,F
                    call                READ_EE
                    movff               ee_data, freq_ins_3
                    incf                ee_adr,F
                    call                READ_EE
                    movff               ee_data, freq_ins_4
                    incf                ee_adr,F
                    call                READ_EE
                    movff               ee_data, freq_ins_5
                    incf                ee_adr,F
                    call                READ_EE
                    movff               ee_data, freq_ins_6
                    incf                ee_adr,F
                    call                READ_EE
                    movff               ee_data, phase_h
                    incf                ee_adr,F
                    call                READ_EE
                    movff               ee_data, vpp
                    incf                ee_adr,F
                    call                READ_EE
                    movff               ee_data, pot_offset_sin
                    incf                ee_adr,F
                    call                READ_EE
                    movff               ee_data, pot_offset_squ

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       SAVE_START_VALUE
;   Input:          shape, freq_ins_0, freq_ins_1, freq_ins_2, freq_ins_3, freq_ins_4, freq_ins_5, freq_ins_6, phase_h, vpp
;   Overview:       Salva i dati relativi alla forma d'onda desiderata all'avvio
;**************************************************************************************************************************************************
;
SAVE_START_VALUE    movlw               h'02'                                           ;imposto il primo indirizzo da utilizzare in EEPROM
                    movwf               ee_adr

                    movff               shape, ee_data                                  ;salvo ogni dato nella rispettiva locazione EEPROM
                    call                WRITE_EE
                    incf                ee_adr,F
                    movff               freq_ins_0, ee_data
                    call                WRITE_EE
                    incf                ee_adr,F
                    movff               freq_ins_1, ee_data
                    call                WRITE_EE
                    incf                ee_adr,F
                    movff               freq_ins_2, ee_data
                    call                WRITE_EE
                    incf                ee_adr,F
                    movff               freq_ins_3, ee_data
                    call                WRITE_EE
                    incf                ee_adr,F
                    movff               freq_ins_4, ee_data
                    call                WRITE_EE
                    incf                ee_adr,F
                    movff               freq_ins_5, ee_data
                    call                WRITE_EE
                    incf                ee_adr,F
                    movff               freq_ins_6, ee_data
                    call                WRITE_EE
                    incf                ee_adr,F
                    movff               phase_h, ee_data
                    call                WRITE_EE
                    incf                ee_adr,F
                    movff               vpp, ee_data
                    call                WRITE_EE

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       TEST_RESET
;   Overview:       Controlla dove  avvenuto il reset ed espone su LCD
;   Output:         Salva in locazione EE_reset_status (0x10) della EEPROM lo stato dei reset in ordine D7/D0: STKFUL/STKUNF/0/RI/TO/PD/POR/BOR
;   Codici LCD:     1: reset da Watch dof timer
;                   2: reset da power on reset
;                   3: reset da Brown-out reset
;                   4: reset da instrucntion reset
;                   5: reset stack full
;                   6: reset stack underfull
;                   7: reset MCLR
;
;**************************************************************************************************************************************************
;
TEST_RESET          movf                RCON,W                                          ;salvo RCON in W
                    comf                WREG, W                                         ;lo nego per avere 1 dove  avvenuto il reset
                    andlw               b'00011111'                                     ;elimino i bit che non servono
                    movwf               reset_status                                    ;salvo il risultato in reset_status
                    movf                STKPTR,W                                        ;leggo i dati relativi ai reset da stack
                    andlw               b'11000000'
                    addwf               reset_status,F                                  ;e li aggiungo a reset_status
                    
                    movff               reset_status,ee_data                            ;salvo il dato in EEPROM
                    movlw               EE_reset_status
                    movwf               ee_adr
                    call                WRITE_EE

                    btfss               RCON,POR                                        ;POR ?
                    goto                POR_RES                                         ;si - gestione POR
                    btfss               RCON,BOR                                        ;BOR ?
                    goto                BOR_RES                                         ;si - gestione BOR
                    btfss               RCON,RI                                         ;Istruzione reset ?
                    goto                RI_RES                                          ;si - gestione
                    btfsc               STKPTR,STKFUL                                   ;Stack overfull ?
                    goto                SFULL_RES                                       ;si - gestione stack over full
                    btfsc               STKPTR,STKUNF                                   ;Stack underfull ?
                    goto                SUNF_RES                                        ;si - gestione stack under full
                    btfss               RCON,TO                                         ;TO = 0 ?
                    btfss               RCON,PD                                         ;si - PD = 1 ?
                    goto                MCLR_RES                                        ;PD = 0 o TO = 1 -> MCLR
                                                                                        ;Reset da Watchdog
                                                                                        ;Nessuno delle precedenti condizioni: reset da watch dog
WDT_RES             call                EXPO_RESET
                    movlw               d'1'
                    call                LCD_SEND_NUMBER
                    goto                OUT_RESET

POR_RES             call                EXPO_RESET
                    movlw               d'2'
                    call                LCD_SEND_NUMBER
                    goto                OUT_RESET

BOR_RES             call                EXPO_RESET
                    movlw               d'3'
                    call                LCD_SEND_NUMBER
                    goto                OUT_RESET

RI_RES              call                EXPO_RESET
                    movlw               d'4'
                    call                LCD_SEND_NUMBER
                    goto                OUT_RESET

SFULL_RES           call                EXPO_RESET
                    movlw               d'5'
                    call                LCD_SEND_NUMBER
                    goto                OUT_RESET

SUNF_RES            call                EXPO_RESET
                    movlw               d'6'
                    call                LCD_SEND_NUMBER
                    goto                OUT_RESET

MCLR_RES            call                EXPO_RESET
                    movlw               d'7'
                    call                LCD_SEND_NUMBER
                    goto                OUT_RESET


OUT_RESET           btfss               ENTER                                           ;attendo pressione del tasto ENTER
                    goto                OUT_RESET

                    bsf                 RCON,POR                                        ;reset flag di reset
                    bsf                 RCON,BOR
                    bsf                 RCON,RI
                    bcf                 STKPTR,STKFUL
                    bcf                 STKPTR,STKUNF
                    clrwdt

                    return
;
;**************************************************************************************************************************************************
;   FUNCTION:       RESET_VAR_MAIN
;   Overview:       Resetta tutte le variabili utilizzate nel programma principale
;**************************************************************************************************************************************************
;
RESET_VAR_MAIN		clrf                freq_hex_s_0
                    clrf                freq_hex_s_1
                    clrf                freq_hex_s_2
                    clrf                freq_hex_s_3
                    clrf				freq_ins_0
					clrf				freq_ins_1
					clrf				freq_ins_2
					clrf				freq_ins_3
					clrf				freq_ins_4
					clrf				freq_ins_5
					clrf				freq_ins_6
                    clrf                count
                    clrf				temp
					clrf				shape
                    clrf                dss_ctrl_l
                    clrf                dss_ctrl_h
                    clrf                phase_h
                    clrf                vpp
                    clrf                mux_vpp
                    clrf                pot_offset_sin
                    clrf                pot_offset_squ
                    clrf                pot_offset

                    return
;
;**************************************************************************************************************************************************
;File esterni
;**************************************************************************************************************************************************
;
					include				"LCD_4bit.asm"                                  ;libreria gestione LCD 4bit
					include				"Delay_32MHz"                                   ;libreria ritardi a 32Mhz
                    include             "Memory_and_table.asm"                          ;libreria scrittura/lettura EEPROM e tabello
                    include				"Calc.asm"                                      ;sottoprogrammi calcoli
                    include             "SPI.asm"                                       ;sottoprogrammi gestione SPI
                    include             "Expos.asm"                                     ;sottoprogrammi scritture su LCD

					end


